searchNodes=[{"ref":"ExToErl.html","title":"ExToErl","type":"module","doc":"Utilities to convert Elixir expressions into the corresponding Erlang. This package is meant to be used as a learning tool or as part of development workflow. It was written to answer questions like: &quot;What does this Elixir expression compile to?&quot;. It&#39;s very useful to explore the output of the Elixir compiler in a user-friendly way. One should be careful when using this in production with user supplied input because most functions in this module run the Elixir compiler and generate atoms dynamically at runtime (as the Elixir compiler does). The code might also be victim of race conditions (I haven&#39;t tested running it in parallel, though). It has no tests yet, but I hope it will have some in the future. The API will probably change a lot. I might switch from raising errors to returning {:ok, value} and :error."},{"ref":"ExToErl.html#beam_to_erlang_abstract_code/1","title":"ExToErl.beam_to_erlang_abstract_code/1","type":"function","doc":"Extracts the Erlang abstract code from a BEAM module. The argument to this function can be either: The module name (an atom) A {:module, module, binary, _} tuple, returned by Module.create/3 The binary part from the tuple above Examples TODO"},{"ref":"ExToErl.html#beam_to_erlang_source/1","title":"ExToErl.beam_to_erlang_source/1","type":"function","doc":"Extracts the Erlang abstract code from a BEAM module and converts it into Erlang source code. The argument to this function can be either: The module name (an atom) A {:module, module, binary, _} tuple, returned by Module.create/3 The binary part from the tuple above Examples iex&gt; module = Module.create(MyModule, quote(do: def f(x) do x end), __ENV__) {:module, MyModule, &lt;&lt;70, 79, 82, 49, 0, 0, 3, 220, 66, 69, 65, 77, 65, 116, 85, 56, 0, 0, 0, 124, 0, 0, 0, 13, 15, 69, 108, 105, 120, 105, 114, 46, 77, 121, 77, 111, 100, 117, 108, 101, 8, 95, 95, 105, 110, 102, 111, ...&gt;&gt;, {:f, 1}} iex&gt; ExToErl.beam_to_erlang_abstract_code(module) [ {:attribute, 6, :file, {&#39;iex&#39;, 6}}, {:attribute, 6, :module, MyModule}, {:attribute, 6, :compile, [:no_auto_import]}, {:attribute, 6, :export, [__info__: 1, f: 1]}, {:attribute, 6, :spec, {{:__info__, 1}, [ {:type, 6, :fun, [ {:type, 6, :product, [ {:type, 6, :union, [ {:atom, 6, :attributes}, {:atom, 6, :compile}, {:atom, 6, :functions}, {:atom, 6, :macros}, {:atom, 6, :md5}, {:atom, 6, :module}, {:atom, 6, :deprecated} ]} ]}, {:type, 6, :any, []} ]} ]}}, {:function, 0, :__info__, 1, [ {:clause, 0, [{:atom, 0, :module}], [], [{:atom, 0, MyModule}]}, {:clause, 0, [{:atom, 0, :functions}], [], [{:cons, 0, {:tuple, 0, [{:atom, 0, :f}, {:integer, 0, 1}]}, {nil, 0}}]}, {:clause, 0, [{:atom, 0, :macros}], [], [nil: 0]}, {:clause, 0, [{:match, 0, {:var, 0, :Key}, {:atom, 0, :attributes}}], [], [ {:call, 0, {:remote, 0, {:atom, 0, :erlang}, {:atom, 0, :get_module_info}}, [{:atom, 0, MyModule}, {:var, 0, :Key}]} ]}, {:clause, 0, [{:match, 0, {:var, 0, :Key}, {:atom, 0, :compile}}], [], [ {:call, 0, {:remote, 0, {:atom, 0, :erlang}, {:atom, 0, :get_module_info}}, [{:atom, 0, MyModule}, {:var, 0, :Key}]} ]}, {:clause, 0, [{:match, 0, {:var, 0, :Key}, {:atom, 0, :md5}}], [], [ {:call, 0, {:remote, 0, {:atom, 0, :erlang}, {:atom, 0, :get_module_info}}, [{:atom, 0, MyModule}, {:var, 0, :Key}]} ]}, {:clause, 0, [{:atom, 0, :deprecated}], [], [nil: 0]} ]}, {:function, 6, :f, 1, [{:clause, 6, [{:var, 6, :__@1}], [], [{:var, 6, :__@1}]}]} ]"},{"ref":"ExToErl.html#beam_to_erlang_source/2","title":"ExToErl.beam_to_erlang_source/2","type":"function","doc":"Extracts the Erlang abstract code from a BEAM module, converts it into Erlang source code and writes it into a file. The first argument to this function can be either: The module name (an atom) A {:module, module, binary, _} tuple, returned by Module.create/3 The binary part from the tuple above Examples iex&gt; module = Module.create(MyModule, quote(do: def f(x) do x end), __ENV__) {:module, MyModule, &lt;&lt;70, 79, 82, 49, 0, 0, 3, 220, 66, 69, 65, 77, 65, 116, 85, 56, 0, 0, 0, 124, 0, 0, 0, 13, 15, 69, 108, 105, 120, 105, 114, 46, 77, 121, 77, 111, 100, 117, 108, 101, 8, 95, 95, 105, 110, 102, 111, ...&gt;&gt;, {:f, 1}} iex&gt; ExToErl.beam_to_erlang_source(module) |&gt; IO.puts() -file(&quot;iex&quot;, 3). -module(&#39;Elixir.MyModule&#39;). -compile([no_auto_import]). -export([&#39;__info__&#39;/1, f/1]). -spec &#39;__info__&#39;(attributes | compile | functions | macros | md5 | module | deprecated) -&gt; any(). &#39;__info__&#39;(module) -&gt; &#39;Elixir.MyModule&#39;; &#39;__info__&#39;(functions) -&gt; [{f, 1}]; &#39;__info__&#39;(macros) -&gt; []; &#39;__info__&#39;(Key = attributes) -&gt; erlang:get_module_info(&#39;Elixir.MyModule&#39;, Key); &#39;__info__&#39;(Key = compile) -&gt; erlang:get_module_info(&#39;Elixir.MyModule&#39;, Key); &#39;__info__&#39;(Key = md5) -&gt; erlang:get_module_info(&#39;Elixir.MyModule&#39;, Key); &#39;__info__&#39;(deprecated) -&gt; []. f(__@1) -&gt; __@1. :ok"},{"ref":"ExToErl.html#elixir_ast_to_erlang_abstract_code/1","title":"ExToErl.elixir_ast_to_erlang_abstract_code/1","type":"function","doc":"Converts Elixir AST into Erlang abstract code. This function expects an Elixir expression. If you supply a block (which is a valid Elixir expression), only the last one will be converted into an Erlang expression. This limitation is a result of the fact that in Erlang a sequence of instructions if not a an Erlang expression (on the other hand, a sequence of Elixir expressions is an Elixir expression). As with most functions in this module, this function creates atoms at runtime because valid Erlang AST contains atoms. Examples iex&gt; ExToErl.elixir_ast_to_erlang_abstract_code({:+, [line: 1], [{:a, [line: 1], nil}, {:b, [line: 1], nil}]}) {:op, 1, :+, {:var, 1, :_a@1}, {:var, 1, :_b@1}} iex&gt; Code.string_to_quoted!(&quot;a - b&quot;) |&gt; ExToErl.elixir_ast_to_erlang_abstract_code() {:op, 1, :-, {:var, 1, :_a@1}, {:var, 1, :_b@1}}"},{"ref":"ExToErl.html#elixir_source_to_erlang_abstract_code/1","title":"ExToErl.elixir_source_to_erlang_abstract_code/1","type":"function","doc":"Converts a string containing Elixir code into an Erlang expression. This function expects an Elixir expression. If you supply a block (which is a valid Elixir expression), only the last one will be converted into an Erlang expression. This limitation is a result of the fact that in Erlang a sequence of instructions if not a an Erlang expression (on the other hand, a sequence of Elixir expressions is an Elixir expression). Don&#39;t use this function to convert entire Elixir modules to Erlang. Use ExToErl.beam_to_erlang_source/1 instead. The function raises if the string is not valid Elixir. As with most functions in this module, this function creates atoms at runtime because valid Erlang AST contains atoms. Examples Single expressions: iex&gt; ExToErl.elixir_source_to_erlang_abstract_code(&quot;a + b&quot;) {:op, 1, :+, {:var, 1, :_a@1}, {:var, 1, :_b@1}} iex&gt; ExToErl.elixir_source_to_erlang_abstract_code(&quot;a &lt;= b&quot;) {:op, 1, :&quot;=&lt;&quot;, {:var, 1, :_a@1}, {:var, 1, :_b@1}} Elixir blocks (only the last expression is returned): iex&gt; ExToErl.elixir_source_to_erlang_abstract_code(&quot;_ = a + b; c + d&quot;) {:op, 1, :+, {:var, 1, :_c@1}, {:var, 1, :_d@1}} You can import functions and macros inside your Elixir expression: iex&gt; ExToErl.elixir_source_to_erlang_abstract_code(&quot;import Bitwise; a &gt;&gt;&gt; b&quot;) {:op, 1, :bsr, {:var, 1, :_a@1}, {:var, 1, :_b@1}} iex&gt; ExToErl.elixir_source_to_erlang_abstract_code(&quot;import Bitwise; a &amp;&amp;&amp; b&quot;) {:op, 1, :band, {:var, 1, :_a@1}, {:var, 1, :_b@1}} Some expressions may raise warnings, although they should be the same wanings as if the Elixir expression were to be compiled inside a normal Elixir module: iex&gt; ExToErl.elixir_source_to_erlang_abstract_code(&quot;a = b&quot;) warning: variable &quot;a&quot; is unused warning: variable &quot;a&quot; is unused {:match, 1, {:var, 1, :_a@2}, {:var, 1, :_b@1}} Some Elixir operators are actually macros or special forms which can be expanded into quite complex Erlang code: iex&gt; ExToErl.elixir_source_to_erlang_abstract_code(&quot;a or b&quot;) {:case, 1, {:var, 1, :_a@1}, [ {:clause, [generated: true, location: 1], [{:atom, 0, false}], [], [{:var, 1, :_b@1}]}, {:clause, [generated: true, location: 1], [{:atom, 0, true}], [], [{:atom, 0, true}]}, {:clause, [generated: true, location: 1], [{:var, 1, :__@1}], [], [ {:call, 1, {:remote, 1, {:atom, 0, :erlang}, {:atom, 1, :error}}, [{:tuple, 1, [{:atom, 0, :badbool}, {:atom, 0, :or}, {:var, 1, :__@1}]}]} ]} ]}"},{"ref":"ExToErl.html#elixir_source_to_erlang_source/1","title":"ExToErl.elixir_source_to_erlang_source/1","type":"function","doc":"Converts a string containing Elixir code into Erlang source code. This function expects an Elixir expression. If you supply a block (which is a valid Elixir expression), only the last one will be converted into an Erlang expression. This limitation is a result of the fact that in Erlang a sequence of instructions if not a an Erlang expression (on the other hand, a sequence of Elixir expressions is an Elixir expression). Don&#39;t use this function to convert entire Elixir modules to Erlang source code. Use ExToErl.beam_to_erlang_source/1 instead. The function raises if the string is not valid Elixir. As with most functions in this module, this function creates atoms at runtime because valid Erlang AST contains atoms. Examples iex&gt; ExToErl.elixir_source_to_erlang_source(&quot;a&quot;) &quot;_a@1\\n&quot; iex&gt; ExToErl.elixir_source_to_erlang_source(&quot;a + b&quot;) &quot;_a@1 + _b@1\\n&quot; iex&gt; ExToErl.elixir_source_to_erlang_source(&quot;a + b &lt; f.(x)&quot;) &quot;_a@1 + _b@1 &lt; _f@1(_x@1)\\n&quot; iex&gt; ExToErl.elixir_source_to_erlang_source(&quot;a or b&quot;) |&gt; IO.puts() case _a@1 of false -&gt; _b@1; true -&gt; true; __@1 -&gt; erlang:error({badbool, &#39;or&#39;, __@1}) end :ok iex(3)&gt; ExToErl.elixir_source_to_erlang_source(&quot;a.b&quot;) |&gt; IO.puts() case _a@1 of \#{b := __@1} -&gt; __@1; __@1 when erlang:is_map(__@1) -&gt; erlang:error({badkey, b, __@1}); __@1 -&gt; __@1:b() end :ok"},{"ref":"ExToErl.html#erlang_abstract_code_to_string/2","title":"ExToErl.erlang_abstract_code_to_string/2","type":"function","doc":"Pretty prints Erlang abstract code as Erlang source code. Examples TODO"},{"ref":"ExToErl.html#erlang_source_to_abstract_code/1","title":"ExToErl.erlang_source_to_abstract_code/1","type":"function","doc":"Parses an Erlang expression into erlang abstract code. Examples iex&gt; ExToErl.erlang_source_to_abstract_code(&quot;A + B.&quot;) {:op, 1, :+, {:var, 1, :A}, {:var, 1, :B}} iex&gt; ExToErl.erlang_source_to_abstract_code(&quot;A &lt; B.&quot;) {:op, 1, :&lt;, {:var, 1, :A}, {:var, 1, :B}} iex&gt; ExToErl.erlang_source_to_abstract_code(&quot;A + B * C &lt; F + G.&quot;) {:op, 1, :&lt;, {:op, 1, :+, {:var, 1, :A}, {:op, 1, :*, {:var, 1, :B}, {:var, 1, :C}}}, {:op, 1, :+, {:var, 1, :F}, {:var, 1, :G}}} iex&gt; ExToErl.erlang_source_to_abstract_code(&quot;A + B * C &lt; f(x) + g(y).&quot;) {:op, 1, :&lt;, {:op, 1, :+, {:var, 1, :A}, {:op, 1, :*, {:var, 1, :B}, {:var, 1, :C}}}, {:op, 1, :+, {:call, 1, {:atom, 1, :f}, [{:atom, 1, :x}]}, {:call, 1, {:atom, 1, :g}, [{:atom, 1, :y}]}}} iex(9)&gt; ExToErl.erlang_source_to_abstract_code(&quot;A + B * C &lt; f(X) + g(Y).&quot;) {:op, 1, :&lt;, {:op, 1, :+, {:var, 1, :A}, {:op, 1, :*, {:var, 1, :B}, {:var, 1, :C}}}, {:op, 1, :+, {:call, 1, {:atom, 1, :f}, [{:var, 1, :X}]}, {:call, 1, {:atom, 1, :g}, [{:var, 1, :Y}]}}}"}]